@@ -27,7 +27,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_bpf.c 1607 2010-07-16 02:36:48Z salcock $
+ * $Id: format_bpf.c 1822 2013-03-28 03:38:53Z salcock $
  *
  */
 
@@ -67,6 +67,32 @@
  * converted to PCAP or ERF.
  */ 
 
+struct lt_bpf_timeval {
+	uint32_t tv_sec;
+	uint32_t tv_usec;
+};
+
+struct libtrace_bpf_hdr {
+	struct lt_bpf_timeval bh_tstamp;	/* timestamp */
+	uint32_t bh_caplen;		/* capture length */
+	uint32_t bh_datalen;		/* wire length */
+	uint16_t bh_hdrlen;		/* header length (incl padding) */
+};
+
+#ifndef BPF_TIMEVAL
+#define BPF_TIMEVAL timeval
+#endif
+
+struct local_bpf_hdr {
+	struct BPF_TIMEVAL bh_tstamp;
+	uint32_t bh_caplen;
+	uint32_t bh_datalen;
+	uint16_t bh_hdrlen;
+};
+
+#define BPFHDR(x) ((struct libtrace_bpf_hdr *)((x)->header))
+
+#if HAVE_DECL_BIOCSETIF
 /* "Global" data that is stored for each BPF input trace */
 struct libtrace_format_data_t {
 	/* The file descriptor that is being captured from */
@@ -94,7 +120,6 @@ struct libtrace_format_data_t {
 
 #define FORMATIN(x) ((struct libtrace_format_data_t*)((x->format_data)))
 
-#define BPFHDR(x) ((struct bpf_hdr *)((x)->header))
 
 /* Attempts to determine if a given filename could refer to a BPF interface */
 static int bpf_probe_filename(const char *filename)
@@ -336,16 +361,21 @@ static int bpf_config_input(libtrace_t *libtrace,
 		 * unimplementable
 		 */
 	}
-	trace_set_err(libtrace,TRACE_ERR_UNKNOWN_OPTION,
-			"Unknown option %i", option);
 	return -1;
 }
 
+#endif  /* HAVE_DECL_BIOCSETIF */
+
 /* Converts a buffer containing a recently read BPF packet record into a
  * libtrace packet */
-static int bpf_prepare_packet(libtrace_t *libtrace, libtrace_packet_t *packet,
+static int bpf_prepare_packet(libtrace_t *libtrace UNUSED, 
+		libtrace_packet_t *packet,
 		void *buffer, libtrace_rt_types_t rt_type, uint32_t flags) {
         
+	struct local_bpf_hdr orig;
+	struct local_bpf_hdr *ptr;
+	struct libtrace_bpf_hdr *replace;
+
 	/* If the packet previously owned a buffer that is not the buffer
 	 * that contains the new packet data, we're going to need to free the
 	 * old one to avoid memory leaks */
@@ -365,23 +395,51 @@ static int bpf_prepare_packet(libtrace_t *libtrace, libtrace_packet_t *packet,
         packet->header = buffer;
 	packet->type = rt_type;
 
+	/* FreeBSD is stupid and uses a timeval in the bpf header
+	 * structure. This means that sometimes our timestamp consumes
+	 * 8 bytes and sometimes it consumes 16 bytes.
+	 *
+	 * Let's try to standardise our header a bit, hopefully without
+	 * overwriting anything else important */
+
+	if (sizeof(struct BPF_TIMEVAL) != sizeof(struct lt_bpf_timeval)) { 	
+		
+		ptr = ((struct local_bpf_hdr *)(packet->header));
+		replace = ((struct libtrace_bpf_hdr *)(packet->header));
+		orig = *ptr;
+
+		replace->bh_tstamp.tv_sec = (uint32_t) (orig.bh_tstamp.tv_sec & 0xffffffff);
+		replace->bh_tstamp.tv_usec = (uint32_t) (orig.bh_tstamp.tv_usec & 0xffffffff);
+		replace->bh_caplen = orig.bh_caplen;
+		replace->bh_datalen = orig.bh_datalen;
+		replace->bh_hdrlen = orig.bh_hdrlen;
+
+
+	}
+
 	/* Find the payload */
 	/* TODO: Pcap deals with a padded FDDI linktype here */
 	packet->payload=(char *)buffer + BPFHDR(packet)->bh_hdrlen;
 
+	/*
 	if (libtrace->format_data == NULL) {
 		if (bpf_init_input(libtrace))
 			return -1;
 	}
+	*/
 
 	return 0;
 }
 
+#if HAVE_DECL_BIOCSETIF
+
 /* Reads the next packet record from a BPF interface and writes it into a 
  * libtrace packet */	
 static int bpf_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) 
 {
 	uint32_t flags = 0;
+
+	packet->type = bpf_linktype_to_rt(FORMATIN(libtrace)->linktype);
 	
 	/* Read from the BPF interface into our capture buffer */
 	if (FORMATIN(libtrace)->remaining<=0) {
@@ -416,7 +474,7 @@ static int bpf_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet)
 	/* Update 'packet' to point to the first packet in our capture
 	 * buffer */
 	if (bpf_prepare_packet(libtrace, packet, FORMATIN(libtrace)->bufptr,
-		TRACE_RT_DATA_BPF, flags)) {
+			packet->type, flags)) {
 		return -1;
 	}
 	
@@ -433,11 +491,13 @@ static int bpf_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet)
 	return BPFHDR(packet)->bh_datalen+BPFHDR(packet)->bh_hdrlen;
 }
 
+#endif  /* HAVE_DECL_BIOCSETIF */
+
 /* Returns the linktype for the interface that we are capturing from */
 static libtrace_linktype_t bpf_get_link_type(const libtrace_packet_t *packet) {
 	/* Convert the linktype that we recorded when we started the trace
 	 * into a suitable libtrace linktype */
-	return pcap_linktype_to_libtrace(FORMATIN(packet->trace)->linktype);
+	return pcap_linktype_to_libtrace(rt_to_pcap_linktype(packet->type));
 }
 
 /* Returns the direction for a given BPF packet record */
@@ -484,6 +544,7 @@ static int bpf_get_framing_length(UNUSED
 	return BPFHDR(packet)->bh_hdrlen;
 }
 
+#if HAVE_DECL_BIOCSETIF
 /* Returns the file descriptor that the capture interface is operating on */
 static int bpf_get_fd(const libtrace_t *trace) {
 	return FORMATIN(trace)->fd;
@@ -491,7 +552,7 @@ static int bpf_get_fd(const libtrace_t *trace) {
 
 /* Prints some slightly useful help text for the BPF capture format */
 static void bpf_help() {
-	printf("bpf format module: $Revision: 1607 $\n");
+	printf("bpf format module: $Revision: 1782 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tbpf:\n");
 	printf("\n");
@@ -499,7 +560,7 @@ static void bpf_help() {
 }
 static struct libtrace_format_t bpf = {
 	"bpf",
-	"$Id: format_bpf.c 1607 2010-07-16 02:36:48Z salcock $",
+	"$Id: format_bpf.c 1822 2013-03-28 03:38:53Z salcock $",
 	TRACE_FORMAT_BPF,
 	bpf_probe_filename,	/* probe filename */
 	NULL,			/* probe magic */
@@ -539,6 +600,56 @@ static struct libtrace_format_t bpf = {
 	bpf_help,		/* help */
 	NULL
 };
+#else 	/* HAVE_DECL_BIOCSETIF */
+/* Prints some slightly useful help text for the BPF capture format */
+static void bpf_help() {
+	printf("bpf format module: $Revision: 1782 $\n");
+	printf("Not supported on this host\n");
+	return;
+}
+static struct libtrace_format_t bpf = {
+	"bpf",
+	"$Id: format_bpf.c 1822 2013-03-28 03:38:53Z salcock $",
+	TRACE_FORMAT_BPF,
+	NULL,			/* probe filename */
+	NULL,			/* probe magic */
+	NULL,	 		/* init_input */
+	NULL,			/* config_input */
+	NULL,			/* start_input */
+	NULL,			/* pause_input */
+	NULL,			/* init_output */
+	NULL,			/* config_output */
+	NULL,			/* start_ouput */
+	NULL,			/* fin_input */
+	NULL,			/* fin_output */
+	NULL,			/* read_packet */
+	bpf_prepare_packet, 	/* prepare_packet */
+	NULL,			/* fin_packet */
+	NULL,			/* write_packet */
+	bpf_get_link_type,	/* get_link_type */
+	bpf_get_direction,	/* get_direction */
+	NULL,			/* set_direction */
+	NULL,			/* get_erf_timestamp */
+	bpf_get_timeval,	/* get_timeval */
+	NULL,			/* get_timespec */
+	NULL,			/* get_seconds */
+	NULL,			/* seek_erf */
+	NULL,			/* seek_timeval */
+	NULL,			/* seek_seconds */
+	bpf_get_capture_length,	/* get_capture_length */
+	bpf_get_wire_length,	/* get_wire_length */
+	bpf_get_framing_length,	/* get_framing_length */
+	NULL,			/* set_capture_length */
+	NULL,/* get_received_packets */
+	NULL,			/* get_filtered_packets */
+	NULL,/* get_dropped_packets */
+	NULL,			/* get_captured_packets */
+	NULL,			/* get_fd */
+	NULL,			/* trace_event */
+	bpf_help,		/* help */
+	NULL
+};
+#endif  /* HAVE_DECL_BIOCSETIF */
 
 void bpf_constructor() {
 	register_format(&bpf);
