@@ -27,7 +27,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_legacy.c 1753 2012-03-05 21:30:01Z salcock $
+ * $Id: format_legacy.c 1825 2013-04-19 05:16:13Z salcock $
  *
  */
 
@@ -153,10 +153,12 @@ static time_t trtime(char *s) {
 	tm.tm_yday = 0; /* ignored */
 	tm.tm_isdst = -1; /* forces check for summer time */
 	
+	/*
 	if (getenv("TZ") == NULL) {
-		perror("getenv(TZ)");
+		fprintf(stderr, "Failed to get the current TZ");
 		return (time_t)0;
 	}
+	*/
 	if (putenv("TZ=Pacific/Auckland")) {
 		perror("putenv");
 		return (time_t)0;
@@ -511,7 +513,7 @@ static void legacynzix_help(void) {
 
 static struct libtrace_format_t legacyatm = {
 	"legacyatm",
-	"$Id: format_legacy.c 1753 2012-03-05 21:30:01Z salcock $",
+	"$Id: format_legacy.c 1825 2013-04-19 05:16:13Z salcock $",
 	TRACE_FORMAT_LEGACY_ATM,
 	NULL,				/* probe filename */
 	NULL,				/* probe magic */
@@ -554,7 +556,7 @@ static struct libtrace_format_t legacyatm = {
 
 static struct libtrace_format_t legacyeth = {
 	"legacyeth",
-	"$Id: format_legacy.c 1753 2012-03-05 21:30:01Z salcock $",
+	"$Id: format_legacy.c 1825 2013-04-19 05:16:13Z salcock $",
 	TRACE_FORMAT_LEGACY_ETH,
 	NULL,				/* probe filename */
 	NULL,				/* probe magic */
@@ -597,7 +599,7 @@ static struct libtrace_format_t legacyeth = {
 
 static struct libtrace_format_t legacypos = {
 	"legacypos",
-	"$Id: format_legacy.c 1753 2012-03-05 21:30:01Z salcock $",
+	"$Id: format_legacy.c 1825 2013-04-19 05:16:13Z salcock $",
 	TRACE_FORMAT_LEGACY_POS,
 	NULL,				/* probe filename */
 	NULL,				/* probe magic */
@@ -640,7 +642,7 @@ static struct libtrace_format_t legacypos = {
 
 static struct libtrace_format_t legacynzix = {
 	"legacynzix",
-	"$Id: format_legacy.c 1753 2012-03-05 21:30:01Z salcock $",
+	"$Id: format_legacy.c 1825 2013-04-19 05:16:13Z salcock $",
 	TRACE_FORMAT_LEGACY_NZIX,
 	NULL,				/* probe filename */
 	NULL,				/* probe magic */
