@@ -27,7 +27,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_pcapfile.c 1751 2012-03-02 04:08:30Z salcock $
+ * $Id: format_pcapfile.c 1799 2013-03-06 00:28:51Z salcock $
  *
  */
 
@@ -628,7 +628,18 @@ static int pcapfile_get_wire_length(const libtrace_packet_t *packet) {
 		trace_get_wireless_flags(link, linktype, &flags);
 		if ((flags & TRACE_RADIOTAP_F_FCS) == 0)
 			return swapl(packet->trace,pcapptr->wirelen)+4;
+	} else if (packet->type == pcap_linktype_to_rt(TRACE_DLT_LINUX_SLL)) {
+		libtrace_sll_header_t *sll;
+		sll = (libtrace_sll_header_t *)packet->payload;
+
+		/* Account for FCS when dealing with Ethernet packets that are
+		 * encapsulated in Linux SLL. This should fix the problem
+		 * where the wire lengths differ if we convert the packet to
+		 * ERF */
+		if (ntohs(sll->protocol) == TRACE_ETHERTYPE_LOOPBACK)
+			return swapl(packet->trace,pcapptr->wirelen)+4;
 	}
+
 	return swapl(packet->trace,pcapptr->wirelen);
 }
 
@@ -671,7 +682,7 @@ static struct libtrace_eventobj_t pcapfile_event(libtrace_t *libtrace, libtrace_
 }
 
 static void pcapfile_help(void) {
-	printf("pcapfile format module: $Revision: 1751 $\n");
+	printf("pcapfile format module: $Revision: 1768 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tpcapfile:/path/to/file\n");
 	printf("\tpcapfile:/path/to/file.gz\n");
@@ -682,7 +693,7 @@ static void pcapfile_help(void) {
 
 static struct libtrace_format_t pcapfile = {
 	"pcapfile",
-	"$Id: format_pcapfile.c 1751 2012-03-02 04:08:30Z salcock $",
+	"$Id: format_pcapfile.c 1799 2013-03-06 00:28:51Z salcock $",
 	TRACE_FORMAT_PCAPFILE,
 	NULL,				/* probe filename */
 	pcapfile_probe_magic,		/* probe magic */
