@@ -27,7 +27,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_rt.c 1752 2012-03-05 21:18:59Z salcock $
+ * $Id: format_rt.c 1799 2013-03-06 00:28:51Z salcock $
  *
  */
 
@@ -104,6 +104,8 @@ struct rt_format_data_t {
 	libtrace_t *dummy_erf;
 	libtrace_t *dummy_pcap;
 	libtrace_t *dummy_linux;
+	libtrace_t *dummy_ring;
+	libtrace_t *dummy_bpf;
 };
 
 /* Connects to an RT server 
@@ -197,6 +199,8 @@ static void rt_init_format_data(libtrace_t *libtrace) {
 	RT_INFO->dummy_erf = NULL;
 	RT_INFO->dummy_pcap = NULL;
 	RT_INFO->dummy_linux = NULL;
+	RT_INFO->dummy_ring = NULL;
+	RT_INFO->dummy_bpf = NULL;
 	RT_INFO->pkt_buffer = NULL;
 	RT_INFO->buf_current = NULL;
 	RT_INFO->buf_filled = 0;
@@ -289,6 +293,12 @@ static int rt_fin_input(libtrace_t *libtrace) {
 
 	if (RT_INFO->dummy_linux)
 		trace_destroy_dead(RT_INFO->dummy_linux);
+	
+	if (RT_INFO->dummy_ring)
+		trace_destroy_dead(RT_INFO->dummy_ring);
+
+	if (RT_INFO->dummy_bpf)
+		trace_destroy_dead(RT_INFO->dummy_bpf);
 	free(libtrace->format_data);
         return 0;
 }
@@ -304,7 +314,7 @@ static int rt_fin_input(libtrace_t *libtrace) {
  * in instances where the fragmentation is done magically by the NIC. This
  * is pretty nasty, but also very rare.
  */
-#define RT_BUF_SIZE 10000U
+#define RT_BUF_SIZE (LIBTRACE_PACKET_BUFSIZE * 2)
 
 /* Receives data from an RT server */
 static int rt_read(libtrace_t *libtrace, void **buffer, size_t len, int block) 
@@ -401,6 +411,21 @@ static int rt_set_format(libtrace_t *libtrace, libtrace_packet_t *packet)
 		return 0;	
 	}
 
+	if (packet->type > TRACE_RT_DATA_BPF &&
+			packet->type < TRACE_RT_DATA_BPF_END) {
+
+		if (!RT_INFO->dummy_bpf) {
+			RT_INFO->dummy_bpf = trace_create_dead("bpf:-");
+			/* This may fail on a non-BSD machine */
+			if (trace_is_err(RT_INFO->dummy_bpf)) {
+				trace_perror(RT_INFO->dummy_bpf, "Creating dead bpf trace");
+				return -1;
+			}
+		}
+		packet->trace = RT_INFO->dummy_bpf;
+		return 0;
+	}
+
 	switch (packet->type) {
 		case TRACE_RT_DUCK_2_4:
 		case TRACE_RT_DUCK_2_5:
@@ -418,9 +443,25 @@ static int rt_set_format(libtrace_t *libtrace, libtrace_packet_t *packet)
 		case TRACE_RT_DATA_LINUX_NATIVE:
 			if (!RT_INFO->dummy_linux) {
 				RT_INFO->dummy_linux = trace_create_dead("int:");
+				/* This may fail on a non-Linux machine */
+				if (trace_is_err(RT_INFO->dummy_linux)) {
+					trace_perror(RT_INFO->dummy_linux, "Creating dead int trace");
+					return -1;
+				}
 			}
 			packet->trace = RT_INFO->dummy_linux;
 			break;
+		case TRACE_RT_DATA_LINUX_RING:
+			if (!RT_INFO->dummy_ring) {
+				RT_INFO->dummy_ring = trace_create_dead("ring:");
+				/* This may fail on a non-Linux machine */
+				if (trace_is_err(RT_INFO->dummy_ring)) {
+					trace_perror(RT_INFO->dummy_ring, "Creating dead int trace");
+					return -1;
+				}
+			}
+			packet->trace = RT_INFO->dummy_ring;
+			break;
 		case TRACE_RT_STATUS:
 		case TRACE_RT_METADATA:
 			/* Just use the RT trace! */
@@ -778,7 +819,7 @@ static void rt_help(void) {
 
 static struct libtrace_format_t rt = {
         "rt",
-        "$Id: format_rt.c 1752 2012-03-05 21:18:59Z salcock $",
+        "$Id: format_rt.c 1799 2013-03-06 00:28:51Z salcock $",
         TRACE_FORMAT_RT,
 	NULL,				/* probe filename */
 	NULL,				/* probe magic */
