@@ -27,7 +27,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: libtrace_int.h 1729 2011-09-20 02:21:53Z salcock $
+ * $Id: libtrace_int.h 1799 2013-03-06 00:28:51Z salcock $
  *
  */
 
@@ -40,7 +40,7 @@
  * @author Perry Lorier
  * @author Shane Alcock
  *
- * @version $Id: libtrace_int.h 1729 2011-09-20 02:21:53Z salcock $
+ * @version $Id: libtrace_int.h 1799 2013-03-06 00:28:51Z salcock $
  *
  * All of the structures and functions defined in this header file are intended
  * for internal use within Libtrace only. They should not be exported as part
@@ -276,29 +276,6 @@ enum {
 	TRACE_PREP_DO_NOT_OWN_BUFFER	=0
 };
 
-/** A local definition of an SLL header */
-typedef struct libtrace_sll_header_t {
-	uint16_t pkttype;          	/**< Packet type */
-	uint16_t hatype;           	/**< Link-layer address type */
-	uint16_t halen;            	/**< Link-layer address length */
-	unsigned char addr[8];	 	/**< Link-layer address */
-	uint16_t protocol;         	/**< Protocol */
-} libtrace_sll_header_t;
-
-
-/* SLL packet types */
-
-/** Packet was addressed for the local host */
-#define TRACE_SLL_HOST		0
-/** Packet was addressed for a broadcast address */
-#define TRACE_SLL_BROADCAST 	1
-/** Packet was addressed for a multicast address */
-#define TRACE_SLL_MULTICAST	2
-/** Packet was addressed for another host but was captured by a promiscuous
- * device */
-#define TRACE_SLL_OTHERHOST	3
-/** Packet originated from the local host */
-#define TRACE_SLL_OUTGOING	4
 
 #ifndef PF_RULESET_NAME_SIZE
 #define PF_RULESET_NAME_SIZE 16
@@ -812,6 +789,13 @@ libtrace_dlt_t libtrace_to_pcap_dlt(libtrace_linktype_t type);
  */
 libtrace_dlt_t rt_to_pcap_linktype(libtrace_rt_types_t rt_type);
 
+/** Converts a PCAP DLT into an RT protocol type for the BPF format.
+ *
+ * @param linktype	The PCAP DLT to be converted
+ * @return The RT type that is equivalent to the provided DLT for BPF
+ */
+libtrace_rt_types_t bpf_linktype_to_rt(libtrace_dlt_t linktype);
+
 /** Converts an ERF type into a libtrace link type.
  *
  * @param erf		The ERF type to be converted
