@@ -27,7 +27,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: protocols_l3.c 1734 2012-01-04 02:35:18Z salcock $
+ * $Id: protocols_l3.c 1811 2013-03-11 00:36:37Z salcock $
  *
  */
 
@@ -35,8 +35,10 @@
 #include "libtrace_int.h"
 #include "libtrace.h"
 #include "protocols.h"
+#include "checksum.h"
 #include <assert.h>
 #include <stdlib.h>
+#include <arpa/inet.h>
 
 #ifdef HAVE_NETPACKET_PACKET_H
 #include <sys/socket.h>
@@ -344,6 +346,51 @@ DLLEXPORT int trace_get_next_option(unsigned char **ptr,int *len,
 	assert(0);
 }
 
+static char *sockaddr_to_string(struct sockaddr *addrptr, char *space,
+		int spacelen) {
+
+	assert(addrptr && space);
+	assert(spacelen > 0);
+	
+	if (addrptr->sa_family == AF_INET) {
+		struct sockaddr_in *v4 = (struct sockaddr_in *)addrptr;
+		inet_ntop(AF_INET, &(v4->sin_addr), space, spacelen);
+	}
+
+	else if (addrptr->sa_family == AF_INET6) {
+		struct sockaddr_in6 *v6 = (struct sockaddr_in6 *)addrptr;
+		inet_ntop(AF_INET6, &(v6->sin6_addr), space, spacelen);
+	}
+#ifdef HAVE_NETPACKET_PACKET_H
+	else if (addrptr->sa_family == AF_PACKET) {
+		struct sockaddr_ll *l2addr = (struct sockaddr_ll *)addrptr;
+		uint8_t *macbytes = (uint8_t *)l2addr->sll_addr;
+
+		snprintf(space, spacelen, "%02x:%02x:%02x:%02x:%02x:%02x",
+				macbytes[0], macbytes[1], macbytes[2],
+				macbytes[3], macbytes[4], macbytes[5]);
+
+	}
+#else
+	else if (addrptr->sa_family == AF_LINK) {
+		struct sockaddr_dl *l2addr = (struct sockaddr_dl *)addrptr;
+		uint8_t *macbytes = (uint8_t *)l2addr->sdl_data;
+
+		snprintf(space, spacelen, "%02x:%02x:%02x:%02x:%02x:%02x",
+				macbytes[0], macbytes[1], macbytes[2],
+				macbytes[3], macbytes[4], macbytes[5]);
+	
+	}
+#endif
+	else {
+		space[0] = '\0';
+		return NULL;
+	}
+
+	return space;
+
+}
+
 /* Extract the source mac address from a frame and bundle it up into a sockaddr */
 static struct sockaddr *get_source_ethernet_address(
 	libtrace_ether_t *ethernet, struct sockaddr *addr)
@@ -460,6 +507,7 @@ DLLEXPORT struct sockaddr *trace_get_source_address(
 				addr6->sin6_port=0;
 			addr6->sin6_flowinfo=0;
 			addr6->sin6_addr=ip6->ip_src;
+			addr6->sin6_scope_id = 0;
 			return addr;
 		}
 		default:
@@ -468,6 +516,27 @@ DLLEXPORT struct sockaddr *trace_get_source_address(
 }
 
 
+DLLEXPORT char *trace_get_source_address_string(
+		const libtrace_packet_t *packet, char *space, int spacelen) {
+
+	static char staticspace[INET6_ADDRSTRLEN];
+	struct sockaddr_storage addr;
+	struct sockaddr *addrptr;
+	
+
+	if (space == NULL || spacelen == 0) {
+		space = staticspace;
+		spacelen = INET6_ADDRSTRLEN;
+	}
+
+	addrptr = trace_get_source_address(packet, (struct sockaddr *)&addr);
+
+	if (addrptr == NULL)
+		return NULL;
+	
+	return sockaddr_to_string(addrptr, space, spacelen);
+}
+
 static struct sockaddr *get_destination_ethernet_address(
 	libtrace_ether_t *ethernet, struct sockaddr *addr)
 {
@@ -586,4 +655,71 @@ DLLEXPORT struct sockaddr *trace_get_destination_address(
 	}
 }
 
+DLLEXPORT char *trace_get_destination_address_string(
+		const libtrace_packet_t *packet, char *space, int spacelen) {
+
+	struct sockaddr_storage addr;
+	struct sockaddr *addrptr;
+	
+	static char staticspace[INET6_ADDRSTRLEN];
+
+	if (space == NULL || spacelen == 0) {
+		space = staticspace;
+		spacelen = INET6_ADDRSTRLEN;
+	}
+
+	addrptr = trace_get_destination_address(packet, 
+			(struct sockaddr *)&addr);
+
+	if (addrptr == NULL)
+		return NULL;
+	
+	return sockaddr_to_string(addrptr, space, spacelen);
+}
+
+DLLEXPORT uint16_t *trace_checksum_layer3(libtrace_packet_t *packet, 
+		uint16_t *csum) {
+
+	void *l3;
+	uint16_t ethertype;
+	uint32_t remaining;
+	uint16_t *csum_ptr;
+
+	uint8_t safety[65536];
+
+	if (csum == NULL)
+		return NULL;
+	
+	l3 = trace_get_layer3(packet, &ethertype, &remaining);
+		
+	if (l3 == NULL)
+		return NULL;
+	
+	if (ethertype == TRACE_ETHERTYPE_IP) {
+		libtrace_ip_t *ip = (libtrace_ip_t *)l3;
+		if (remaining < sizeof(libtrace_ip_t))
+			return NULL;
+
+		csum_ptr = &ip->ip_sum;
+
+		/* I hate memcpys, but this is the only truly safe way to
+		 * do this without modifying the packet. I'm trying to be
+		 * careful about not creating any more thread-safety issues
+		 * than there already are :) */
+		memcpy(safety, ip, ip->ip_hl * sizeof(uint32_t));
+		
+		/* Set the checksum to zero, so we can do the calculation */
+		ip = (libtrace_ip_t *)safety;
+		ip->ip_sum = 0;
+
+		*csum = checksum_buffer(safety, ip->ip_hl * sizeof(uint32_t));
+		
+		/* Remember to byteswap appropriately */
+		*csum = ntohs(*csum);
+		
+		return csum_ptr;
+	}
+
+	return NULL;
+}
 
