@@ -27,7 +27,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: trace.c 1752 2012-03-05 21:18:59Z salcock $
+ * $Id: trace.c 1799 2013-03-06 00:28:51Z salcock $
  *
  */
 
@@ -135,15 +135,11 @@ static void trace_init(void)
 		tsh_constructor();
 		legacy_constructor();
 		atmhdr_constructor();
-#ifdef HAVE_NETPACKET_PACKET_H
 		linuxnative_constructor();
-#endif
 #ifdef HAVE_LIBPCAP
 		pcap_constructor();
 #endif
-#if HAVE_DECL_BIOCSETIF
 		bpf_constructor();
-#endif
 		pcapfile_constructor();
 		rt_constructor();
 #ifdef HAVE_DAG
@@ -586,13 +582,24 @@ DLLEXPORT void trace_destroy(libtrace_t *libtrace) {
 	if (libtrace->format) {
 		if (libtrace->started && libtrace->format->pause_input)
 			libtrace->format->pause_input(libtrace);
-		libtrace->format->fin_input(libtrace);
+		if (libtrace->format->fin_input)
+			libtrace->format->fin_input(libtrace);
 	}
         /* Need to free things! */
         if (libtrace->uridata)
 		free(libtrace->uridata);
-	if (libtrace->event.packet)
-		trace_destroy_packet(libtrace->event.packet);
+	if (libtrace->event.packet) {
+		/* Don't use trace_destroy_packet here - there is almost
+		 * certainly going to be another libtrace_packet_t that is
+		 * pointing to the buffer for this packet, so we don't want
+		 * to free it. Rather, it will get freed when the user calls
+		 * trace_destroy_packet on the libtrace_packet_t that they
+		 * own.
+		 *
+		 * All we need to do then is free our packet structure itself.
+		 */
+		 free(libtrace->event.packet);
+	}
         free(libtrace);
 }
 
@@ -663,7 +670,7 @@ DLLEXPORT libtrace_packet_t *trace_copy_packet(const libtrace_packet_t *packet)
 /** Destroy a packet object
  */
 DLLEXPORT void trace_destroy_packet(libtrace_packet_t *packet) {
-	if (packet->buf_control == TRACE_CTRL_PACKET) {
+	if (packet->buf_control == TRACE_CTRL_PACKET && packet->buffer) {
 		free(packet->buffer);
 	}
 	packet->buf_control=(buf_control_t)'\0'; 
